@@ -1,15 +1,12 @@
 """HTTP Client for asyncio."""
 
-__all__ = ['request', 'HttpClient']
+__all__ = ['request']
 
 import asyncio
-import collections
 import http.cookies
 import io
 import json
 import mimetypes
-import random
-import time
 import urllib.parse
 import weakref
 import warnings
@@ -20,7 +17,7 @@ except ImportError:  # pragma: no cover
 
 import aiohttp
 from . import helpers, streams
-from .log import client_log
+from .log import client_logger
 from .streams import EOF_MARKER, FlowControlStreamReader
 from .multidict import CaseInsensitiveMultiDict, MultiDict, MutableMultiDict
 
@@ -104,9 +101,8 @@ def request(method, url, *,
             auth=auth, version=version, compress=compress, chunked=chunked,
             loop=loop, expect100=expect100, response_class=response_class)
 
+        conn = yield from connector.connect(req)
         try:
-            conn = yield from connector.connect(req)
-
             resp = req.send(conn.writer, conn.reader)
             try:
                 yield from resp.start(conn, read_until_eof)
@@ -114,10 +110,11 @@ def request(method, url, *,
                 resp.close()
                 conn.close()
                 raise
-        except aiohttp.BadStatusLine as exc:
-            raise aiohttp.ClientConnectionError(exc)
+        except (aiohttp.HttpProcessingError,
+                aiohttp.ServerDisconnectedError) as exc:
+            raise aiohttp.ClientResponseError(exc)
         except OSError as exc:
-            raise aiohttp.OsConnectionError(exc)
+            raise aiohttp.ClientOSError(exc)
 
         # redirects
         if resp.status in (301, 302, 303, 307) and allow_redirects:
@@ -405,7 +402,7 @@ class ClientRequest:
                 self.headers['CONTENT-TYPE'] = data.content_type
 
             if data.is_multipart:
-                self.chunked = self.chunked or 8196
+                self.chunked = self.chunked or 8192
             else:
                 if 'CONTENT-LENGTH' not in self.headers and not self.chunked:
                     self.headers['CONTENT-LENGTH'] = str(len(self.body))
@@ -420,10 +417,10 @@ class ClientRequest:
             if 'chunked' not in te:
                 self.headers['TRANSFER-ENCODING'] = 'chunked'
 
-            self.chunked = self.chunked if type(self.chunked) is int else 8196
+            self.chunked = self.chunked if type(self.chunked) is int else 8192
         else:
             if 'chunked' in te:
-                self.chunked = 8196
+                self.chunked = 8192
             else:
                 self.chunked = None
                 if 'CONTENT-LENGTH' not in self.headers:
@@ -510,7 +507,7 @@ class ClientRequest:
                     request.write(chunk)
         except Exception as exc:
             reader.set_exception(
-                aiohttp.ClientConnectionError(
+                aiohttp.ClientRequestError(
                     'Can not write request body for %s' % self.url))
         else:
             try:
@@ -522,7 +519,7 @@ class ClientRequest:
                     yield from ret
             except Exception as exc:
                 reader.set_exception(
-                    aiohttp.ClientConnectionError(
+                    aiohttp.ClientRequestError(
                         'Can not write request body for %s' % self.url))
 
         self._writer = None
@@ -659,7 +656,7 @@ class ClientResponse:
                 try:
                     self.cookies.load(hdr)
                 except http.cookies.CookieError as exc:
-                    client_log.warning(
+                    client_logger.warning(
                         'Can not load response cookies: %s', exc)
             connection.share_cookies(self.cookies)
         return self
@@ -758,171 +755,10 @@ class ClientResponse:
 
         ctype = self.headers.get('CONTENT-TYPE', '').lower()
         if 'json' not in ctype:
-            client_log.warning(
+            client_logger.warning(
                 'Attempt to decode JSON with unexpected mimetype: %s', ctype)
 
         if not self._content.strip():
             return None
 
         return loads(self._content.decode(self._get_encoding(encoding)))
-
-
-class HttpClient:
-    """Allow to use multiple hosts with same path. And automatically
-    mark failed hosts.
-    """
-
-    def __init__(self, hosts, *, method=None, path=None, ssl=False,
-                 conn_pool=True, conn_timeout=None, failed_timeout=5.0,
-                 resolve=True, resolve_timeout=360.0, keepalive_timeout=30,
-                 verify_ssl=True, loop=None):
-        super().__init__()
-
-        if isinstance(hosts, str):
-            hosts = (hosts,)
-
-        if not hosts:
-            raise ValueError('Hosts are required')
-
-        self._hosts = []
-        for host in hosts:
-            has_port = False
-            if isinstance(host, str):
-                if ':' in host:
-                    host, port = host.split(':')
-                    try:
-                        port = int(port)
-                        has_port = True
-                    except:
-                        raise ValueError('Port has to be integer: %s' % host)
-                else:
-                    port = 80
-            else:
-                has_port = True
-                host, port = host
-
-            self._hosts.append((host, port, has_port))
-
-        self._method = method
-        self._path = path
-        self._schema = 'https' if ssl else 'http'
-
-        self._failed = collections.deque()
-        self._failed_handle = None
-        self._failed_timeout = failed_timeout
-
-        if loop is None:
-            loop = asyncio.get_event_loop()
-        self._loop = loop
-
-        if conn_pool:
-            self._connector = aiohttp.TCPConnector(
-                share_cookies=True, conn_timeout=conn_timeout,
-                keepalive_timeout=keepalive_timeout,
-                resolve=resolve, verify_ssl=verify_ssl, loop=loop)
-
-            self._resolve_timeout = resolve_timeout
-            self._resolve_handle = self._loop.call_later(
-                self._resolve_timeout, self._cleanup_resolved_host)
-        else:
-            self._connector = None
-
-    def _cleanup_resolved_host(self):
-        if self._connector:
-            self._connector.clear_resolved_hosts()
-            self._resolve_handle.cancel()
-            self._resolve_handle = self._loop.call_later(
-                self._resolve_timeout, self._cleanup_resolved_host)
-
-    def _resurrect_failed(self):
-        now = int(time.time())
-
-        while self._failed:
-            if (now - self._failed[0][1]) >= self._failed_timeout:
-                self._hosts.append(self._failed.popleft()[0])
-            else:
-                break
-
-        if self._failed:
-            self._failed_handle = self._loop.call_later(
-                self._failed_timeout, self._resurrect_failed)
-        else:
-            self._failed_handle = None
-
-    @asyncio.coroutine
-    def request(self, method=None, path=None, *,
-                params=None,
-                data=None,
-                headers=None,
-                cookies=None,
-                files=None,
-                auth=None,
-                allow_redirects=True,
-                max_redirects=10,
-                encoding='utf-8',
-                version=aiohttp.HttpVersion11,
-                compress=None,
-                chunked=None,
-                expect100=False,
-                read_until_eof=True):
-
-        if method is None:
-            method = self._method
-        if path is None:
-            path = self._path
-
-        assert method is not None, 'request "method" is required.'
-        assert path is not None, 'request "path" is required.'
-
-        # if all hosts marked as failed try first from failed
-        if not self._hosts:
-            self._hosts.append(self._failed.popleft()[0])
-
-        url = ''
-        hosts = self._hosts
-
-        while hosts:
-            idx = random.randint(0, len(hosts)-1)
-
-            info = hosts[idx]
-            if info[2]:
-                url = urllib.parse.urljoin(
-                    '{}://{}:{}'.format(self._schema, info[0], info[1]), path)
-            else:
-                url = urllib.parse.urljoin(
-                    '{}://{}'.format(self._schema, info[0]), path)
-
-            try:
-                resp = yield from request(
-                    method, url, params=params, data=data, headers=headers,
-                    cookies=cookies, files=files, auth=auth,
-                    encoding=encoding, allow_redirects=allow_redirects,
-                    version=version, max_redirects=max_redirects,
-                    compress=compress, chunked=chunked,
-                    expect100=expect100, read_until_eof=read_until_eof,
-                    connector=self._connector, loop=self._loop)
-            except (aiohttp.ConnectionError, aiohttp.TimeoutError):
-                pass
-            else:
-                if 500 <= resp.status <= 600:
-                    self._cleanup_resolved_host()
-
-                return resp
-
-            if info in hosts:
-                # could be removed concurrently
-                hosts.remove(info)
-                self._failed.append((info, int(time.time())))
-                if not self._failed_handle:
-                    self._failed_handle = self._loop.call_later(
-                        self._failed_timeout, self._resurrect_failed)
-
-                if self._connector:
-                    self._connector.clear_resolved_hosts(info[0], info[1])
-
-        raise aiohttp.ConnectionError('All hosts are unreachable %s' % url)
-
-
-# backward compatibility
-HttpRequest = ClientRequest
-HttpResponse = ClientResponse
