@@ -38,14 +38,14 @@ class HttpRequestHandler(aiohttp.server.ServerHttpProtocol):
                 isdir = os.path.isdir(path)
 
         if not path:
-            raise aiohttp.HttpErrorException(404)
+            raise aiohttp.HttpProcessingError(404)
 
         for hdr, val in message.headers.items(getall=True):
             print(hdr, val)
 
         if isdir and not path.endswith('/'):
             path = path + '/'
-            raise aiohttp.HttpErrorException(
+            raise aiohttp.HttpProcessingError(
                 302, headers=(('URI', path), ('Location', path)))
 
         response = aiohttp.Response(
@@ -88,10 +88,10 @@ class HttpRequestHandler(aiohttp.server.ServerHttpProtocol):
 
             try:
                 with open(path, 'rb') as fp:
-                    chunk = fp.read(8196)
+                    chunk = fp.read(8192)
                     while chunk:
                         response.write(chunk)
-                        chunk = fp.read(8196)
+                        chunk = fp.read(8192)
             except OSError:
                 response.write(b'Cannot open')
 
