@@ -143,7 +143,7 @@ class BaseConnectorTests(unittest.TestCase):
         self.assertEqual(conn._get(1), (None, None))
 
         tr, proto = unittest.mock.Mock(), unittest.mock.Mock()
-        conn._conns[1] = [(tr, proto, time.time()-1000)]
+        conn._conns[1] = [(tr, proto, time.time() - 1000)]
         self.assertEqual(conn._get(1), (None, None))
         self.assertEqual(conn._conns[1], [])
 
@@ -228,6 +228,27 @@ class BaseConnectorTests(unittest.TestCase):
         self.assertEqual(connection._protocol, proto)
         self.assertIsInstance(connection, Connection)
 
+    def test_connect_timeout(self):
+        conn = aiohttp.BaseConnector(loop=self.loop)
+        conn._create_connection = unittest.mock.Mock()
+        conn._create_connection.return_value = asyncio.Future(loop=self.loop)
+        conn._create_connection.return_value.set_exception(
+            asyncio.TimeoutError())
+
+        with self.assertRaises(aiohttp.ClientTimeoutError):
+            req = unittest.mock.Mock()
+            self.loop.run_until_complete(conn.connect(req))
+
+    def test_connect_oserr(self):
+        conn = aiohttp.BaseConnector(loop=self.loop)
+        conn._create_connection = unittest.mock.Mock()
+        conn._create_connection.return_value = asyncio.Future(loop=self.loop)
+        conn._create_connection.return_value.set_exception(OSError())
+
+        with self.assertRaises(aiohttp.ClientOSError):
+            req = unittest.mock.Mock()
+            self.loop.run_until_complete(conn.connect(req))
+
     def test_start_cleanup_task(self):
         loop = unittest.mock.Mock()
         conn = aiohttp.BaseConnector(loop=loop)
@@ -277,6 +298,20 @@ class BaseConnectorTests(unittest.TestCase):
         self.assertEqual(conn.family, socket.AF_INET)
         self.assertEqual(conn.resolved_hosts, {})
 
+    def test_tcp_connector_clear_resolved_hosts(self):
+        conn = aiohttp.TCPConnector(loop=self.loop)
+        info = object()
+        conn._resolved_hosts[('localhost', 123)] = info
+        conn._resolved_hosts[('localhost', 124)] = info
+        conn.clear_resolved_hosts('localhost', 123)
+        self.assertEqual(
+            conn.resolved_hosts, {('localhost', 124): info})
+        conn.clear_resolved_hosts('localhost', 123)
+        self.assertEqual(
+            conn.resolved_hosts, {('localhost', 124): info})
+        conn.clear_resolved_hosts()
+        self.assertEqual(conn.resolved_hosts, {})
+
 
 class HttpClientConnectorTests(unittest.TestCase):
 
@@ -574,7 +609,7 @@ class ProxyConnectorTests(unittest.TestCase):
 
         req = ClientRequest('GET', 'https://www.python.org')
         with self.assertRaisesRegex(
-                aiohttp.HttpProxyError, "(400, 'bad request')"):
+                aiohttp.HttpProxyError, "400, message='bad request'"):
             self.loop.run_until_complete(connector._create_connection(req))
 
     @unittest.mock.patch('aiohttp.connector.ClientRequest')
