@@ -31,8 +31,8 @@ class TestWebFunctional(unittest.TestCase):
             app.router.add_route(method, path, handler)
 
         port = self.find_unused_port()
-        srv = yield from self.loop.create_server(app.make_handler,
-                                                 '127.0.0.1', port)
+        srv = yield from self.loop.create_server(
+            app.make_handler(), '127.0.0.1', port)
         url = "http://127.0.0.1:{}".format(port) + path
         self.addCleanup(srv.close)
         return app, srv, url
@@ -43,7 +43,7 @@ class TestWebFunctional(unittest.TestCase):
         def handler(request):
             body = yield from request.read()
             self.assertEqual(b'', body)
-            return web.Response(request, b'OK')
+            return web.Response(body=b'OK')
 
         @asyncio.coroutine
         def go():
@@ -59,9 +59,9 @@ class TestWebFunctional(unittest.TestCase):
 
         @asyncio.coroutine
         def handler(request):
-            data = yield from request.POST()
+            data = yield from request.post()
             self.assertEqual({'a': '1', 'b': '2'}, dict(data))
-            return web.Response(request, b'OK')
+            return web.Response(body=b'OK')
 
         @asyncio.coroutine
         def go():
@@ -80,7 +80,7 @@ class TestWebFunctional(unittest.TestCase):
         def handler(request):
             data = yield from request.text()
             self.assertEqual('русский', data)
-            return web.Response(request, data.encode('utf8'))
+            return web.Response(text=data)
 
         @asyncio.coroutine
         def go():
@@ -101,7 +101,7 @@ class TestWebFunctional(unittest.TestCase):
         def handler(request):
             data = yield from request.json()
             self.assertEqual(dct, data)
-            resp = web.Response(request)
+            resp = web.Response()
             resp.content_type = 'application/json'
             resp.body = json.dumps(data).encode('utf8')
             return resp
@@ -123,7 +123,7 @@ class TestWebFunctional(unittest.TestCase):
 
         @asyncio.coroutine
         def handler(request):
-            raise web.HTTPMovedPermanently(request, location='/path')
+            raise web.HTTPMovedPermanently(location='/path')
 
         @asyncio.coroutine
         def go():
@@ -150,11 +150,11 @@ class TestWebFunctional(unittest.TestCase):
 
         @asyncio.coroutine
         def handler(request):
-            data = yield from request.POST()
+            data = yield from request.post()
             self.assertEqual(['sample.crt'], list(data.keys()))
             for fs in data.values():
                 check_file(fs)
-            resp = web.Response(request, b'OK')
+            resp = web.Response(body=b'OK')
             return resp
 
         @asyncio.coroutine
@@ -183,11 +183,11 @@ class TestWebFunctional(unittest.TestCase):
 
         @asyncio.coroutine
         def handler(request):
-            data = yield from request.POST()
+            data = yield from request.post()
             self.assertEqual(['sample.crt', 'sample.key'], list(data.keys()))
             for fs in data.values():
                 check_file(fs)
-            resp = web.Response(request, b'OK')
+            resp = web.Response(body=b'OK')
             return resp
 
         @asyncio.coroutine
@@ -206,7 +206,7 @@ class TestWebFunctional(unittest.TestCase):
             yield from request.release()
             chunk = yield from request.payload.readany()
             self.assertIs(web.EOF_MARKER, chunk)
-            return web.Response(request, b'OK')
+            return web.Response(body=b'OK')
 
         @asyncio.coroutine
         def go():
@@ -220,9 +220,9 @@ class TestWebFunctional(unittest.TestCase):
     def test_POST_DATA_with_content_transfer_encoding(self):
         @asyncio.coroutine
         def handler(request):
-            data = yield from request.POST()
+            data = yield from request.post()
             self.assertEqual(b'123', data['name'])
-            return web.Response(request)
+            return web.Response()
 
         @asyncio.coroutine
         def go():
@@ -256,9 +256,9 @@ class TestWebFunctional(unittest.TestCase):
             ct = resp.headers['CONTENT-TYPE']
             self.assertEqual('application/octet-stream', ct)
 
-            resp = yield from request('GET', url+'fake', loop=self.loop)
+            resp = yield from request('GET', url + 'fake', loop=self.loop)
             self.assertEqual(404, resp.status)
-            resp = yield from request('GET', url+'/../../', loop=self.loop)
+            resp = yield from request('GET', url + '/../../', loop=self.loop)
             self.assertEqual(404, resp.status)
 
         with tempfile.TemporaryDirectory() as tmpdirname:
@@ -273,10 +273,10 @@ class TestWebFunctional(unittest.TestCase):
 
         @asyncio.coroutine
         def handler(request):
-            data = yield from request.POST()
+            data = yield from request.post()
             lst = list(sorted(data.items(getall=True)))
             self.assertEqual([('a', '1'), ('a', '2')], lst)
-            return web.Response(request, b'OK')
+            return web.Response(body=b'OK')
 
         @asyncio.coroutine
         def go():
