@@ -27,30 +27,22 @@ class TestStreamResponse(unittest.TestCase):
         return req
 
     def test_ctor(self):
-        req = self.make_request('GET', '/')
-        resp = StreamResponse(req)
-
-        self.assertEqual(req, resp._request)
+        resp = StreamResponse()
         self.assertEqual(200, resp.status)
-        self.assertTrue(resp.keep_alive)
-        self.assertIs(req, resp.request)
+        self.assertIsNone(resp.keep_alive)
 
     def test_content_length(self):
-        req = self.make_request('GET', '/')
-        resp = StreamResponse(req)
-
+        resp = StreamResponse()
         self.assertIsNone(resp.content_length)
 
     def test_content_length_setter(self):
-        req = self.make_request('GET', '/')
-        resp = StreamResponse(req)
+        resp = StreamResponse()
 
         resp.content_length = 234
         self.assertEqual(234, resp.content_length)
 
     def test_drop_content_length_header_on_setting_len_to_None(self):
-        req = self.make_request('GET', '/')
-        resp = StreamResponse(req)
+        resp = StreamResponse()
 
         resp.content_length = 1
         self.assertEqual("1", resp.headers['Content-Length'])
@@ -58,8 +50,7 @@ class TestStreamResponse(unittest.TestCase):
         self.assertNotIn('Content-Length', resp.headers)
 
     def test_set_content_length_to_None_on_non_set(self):
-        req = self.make_request('GET', '/')
-        resp = StreamResponse(req)
+        resp = StreamResponse()
 
         resp.content_length = None
         self.assertNotIn('Content-Length', resp.headers)
@@ -67,16 +58,13 @@ class TestStreamResponse(unittest.TestCase):
         self.assertNotIn('Content-Length', resp.headers)
 
     def test_setting_content_type(self):
-
-        req = self.make_request('GET', '/')
-        resp = StreamResponse(req)
+        resp = StreamResponse()
 
         resp.content_type = 'text/html'
         self.assertEqual('text/html', resp.headers['content-type'])
 
     def test_setting_charset(self):
-        req = self.make_request('GET', '/')
-        resp = StreamResponse(req)
+        resp = StreamResponse()
 
         resp.content_type = 'text/html'
         resp.charset = 'koi8-r'
@@ -84,22 +72,19 @@ class TestStreamResponse(unittest.TestCase):
                          resp.headers['content-type'])
 
     def test_default_charset(self):
-        req = self.make_request('GET', '/')
-        resp = StreamResponse(req)
+        resp = StreamResponse()
 
         self.assertIsNone(resp.charset)
 
     def test_reset_charset(self):
-        req = self.make_request('GET', '/')
-        resp = StreamResponse(req)
+        resp = StreamResponse()
 
         resp.content_type = 'text/html'
         resp.charset = None
         self.assertIsNone(resp.charset)
 
     def test_reset_charset_after_setting(self):
-        req = self.make_request('GET', '/')
-        resp = StreamResponse(req)
+        resp = StreamResponse()
 
         resp.content_type = 'text/html'
         resp.charset = 'koi8-r'
@@ -107,46 +92,44 @@ class TestStreamResponse(unittest.TestCase):
         self.assertIsNone(resp.charset)
 
     def test_charset_without_content_type(self):
-        req = self.make_request('GET', '/')
-        resp = StreamResponse(req)
+        resp = StreamResponse()
 
         with self.assertRaises(RuntimeError):
             resp.charset = 'koi8-r'
 
-    def test_cannot_send_headers_twice(self):
+    @mock.patch('aiohttp.web.ResponseImpl')
+    def test_start(self, ResponseImpl):
         req = self.make_request('GET', '/')
-        resp = StreamResponse(req)
+        resp = StreamResponse()
+        self.assertIsNone(resp.keep_alive)
 
-        resp.send_headers()
+        msg = resp.start(req)
+
+        self.assertTrue(msg.send_headers.called)
+        self.assertIs(msg, resp.start(req))
+
+        self.assertTrue(resp.keep_alive)
+
+        req2 = self.make_request('GET', '/')
         with self.assertRaises(RuntimeError):
-            resp.send_headers()
-
-    def test_cannot_send_headers_after_eof(self):
-        req = self.make_request('GET', '/')
-        resp = StreamResponse(req)
-
-        resp.send_headers()
-        resp.write_eof()
-        with self.assertRaises(RuntimeError):
-            resp.send_headers()
+            resp.start(req2)
 
     def test_write_non_byteish(self):
-        req = self.make_request('GET', '/')
-        resp = StreamResponse(req)
+        resp = StreamResponse()
+        resp.start(self.make_request('GET', '/'))
 
         with self.assertRaises(TypeError):
             resp.write(123)
 
-    def test_write_before_sending_headers(self):
-        req = self.make_request('GET', '/')
-        resp = StreamResponse(req)
+    def test_write_before_start(self):
+        resp = StreamResponse()
 
-        resp.write(b'data')
-        self.assertTrue(self.writer.write.called)
+        with self.assertRaises(RuntimeError):
+            resp.write(b'data')
 
     def test_cannot_write_after_eof(self):
-        req = self.make_request('GET', '/')
-        resp = StreamResponse(req)
+        resp = StreamResponse()
+        resp.start(self.make_request('GET', '/'))
 
         resp.write(b'data')
         self.writer.drain.return_value = ()
@@ -158,15 +141,14 @@ class TestStreamResponse(unittest.TestCase):
         self.assertFalse(self.writer.write.called)
 
     def test_cannot_write_eof_before_headers(self):
-        req = self.make_request('GET', '/')
-        resp = StreamResponse(req)
+        resp = StreamResponse()
 
         with self.assertRaises(RuntimeError):
             self.loop.run_until_complete(resp.write_eof())
 
     def test_cannot_write_eof_twice(self):
-        req = self.make_request('GET', '/')
-        resp = StreamResponse(req)
+        resp = StreamResponse()
+        resp.start(self.make_request('GET', '/'))
 
         resp.write(b'data')
         self.writer.drain.return_value = ()
@@ -178,28 +160,26 @@ class TestStreamResponse(unittest.TestCase):
         self.assertFalse(self.writer.write.called)
 
     def test_write_returns_drain(self):
-        req = self.make_request('GET', '/')
-        resp = StreamResponse(req)
+        resp = StreamResponse()
+        resp.start(self.make_request('GET', '/'))
 
         self.assertEqual((), resp.write(b'data'))
 
     def test_write_returns_empty_tuple_on_empty_data(self):
-        req = self.make_request('GET', '/')
-        resp = StreamResponse(req)
+        resp = StreamResponse()
+        resp.start(self.make_request('GET', '/'))
 
         self.assertEqual((), resp.write(b''))
 
     def test_force_close(self):
-        req = self.make_request('GET', '/')
-        resp = StreamResponse(req)
+        resp = StreamResponse()
 
-        self.assertTrue(resp.keep_alive)
+        self.assertIsNone(resp.keep_alive)
         resp.force_close()
         self.assertFalse(resp.keep_alive)
 
     def test_response_cookies(self):
-        req = self.make_request('GET', '/')
-        resp = StreamResponse(req)
+        resp = StreamResponse()
 
         self.assertEqual(resp.cookies, {})
         self.assertEqual(str(resp.cookies), '')
@@ -222,8 +202,7 @@ class TestStreamResponse(unittest.TestCase):
                          'Set-Cookie: name=value; Domain=local.host')
 
     def test_response_cookie_path(self):
-        req = self.make_request('GET', '/')
-        resp = StreamResponse(req)
+        resp = StreamResponse()
 
         self.assertEqual(resp.cookies, {})
 
@@ -248,8 +227,7 @@ class TestStreamResponse(unittest.TestCase):
                          'Version=2.0')
 
     def test_response_cookie__issue_del_cookie(self):
-        req = self.make_request('GET', '/')
-        resp = StreamResponse(req)
+        resp = StreamResponse()
 
         self.assertEqual(resp.cookies, {})
         self.assertEqual(str(resp.cookies), '')
@@ -258,8 +236,7 @@ class TestStreamResponse(unittest.TestCase):
         self.assertEqual(str(resp.cookies), 'Set-Cookie: name=; Max-Age=0')
 
     def test_cookie_set_after_del(self):
-        req = self.make_request('GET', '/')
-        resp = StreamResponse(req)
+        resp = StreamResponse()
 
         resp.del_cookie('name')
         resp.set_cookie('name', 'val')
@@ -267,8 +244,7 @@ class TestStreamResponse(unittest.TestCase):
         self.assertEqual(str(resp.cookies), 'Set-Cookie: name=val')
 
     def test_set_status_with_reason(self):
-        req = self.make_request('GET', '/')
-        resp = StreamResponse(req)
+        resp = StreamResponse()
 
         resp.set_status(200, "Everithing is fine!")
         self.assertEqual(200, resp.status)
@@ -296,8 +272,7 @@ class TestResponse(unittest.TestCase):
         return req
 
     def test_ctor(self):
-        req = self.make_request('GET', '/')
-        resp = Response(req)
+        resp = Response()
 
         self.assertEqual(200, resp.status)
         self.assertEqual('OK', resp.reason)
@@ -307,28 +282,62 @@ class TestResponse(unittest.TestCase):
                          resp.headers)
 
     def test_ctor_with_headers_and_status(self):
-        req = self.make_request('GET', '/')
-        resp = Response(req, b'body', status=201, headers={'Age': '12'})
+        resp = Response(body=b'body', status=201, headers={'Age': '12'})
 
         self.assertEqual(201, resp.status)
         self.assertEqual(b'body', resp.body)
         self.assertEqual(4, resp.content_length)
-        self.assertEqual(CaseInsensitiveMultiDict([('CONTENT-LENGTH', '4'),
-                                                   ('AGE', '12')]),
+        self.assertEqual(CaseInsensitiveMultiDict([('AGE', '12'),
+                                                   ('CONTENT-LENGTH', '4')]),
                          resp.headers)
 
+    def test_ctor_content_type(self):
+        resp = Response(content_type='application/json')
+
+        self.assertEqual(200, resp.status)
+        self.assertEqual('OK', resp.reason)
+        self.assertEqual(
+            CaseInsensitiveMultiDict([('CONTENT-TYPE', 'application/json'),
+                                      ('CONTENT-LENGTH', '0')]),
+            resp.headers)
+
+    def test_ctor_text_body_combined(self):
+        with self.assertRaises(ValueError):
+            Response(body=b'123', text='test text')
+
+    def test_ctor_text(self):
+        resp = Response(text='test text')
+
+        self.assertEqual(200, resp.status)
+        self.assertEqual('OK', resp.reason)
+        self.assertEqual(
+            CaseInsensitiveMultiDict(
+                [('CONTENT-TYPE', 'text/plain; charset=utf-8'),
+                 ('CONTENT-LENGTH', '9')]),
+            resp.headers)
+
+        self.assertEqual(resp.body, b'test text')
+        self.assertEqual(resp.text, 'test text')
+
     def test_assign_nonbyteish_body(self):
-        req = self.make_request('GET', '/')
-        resp = Response(req, b'data')
+        resp = Response(body=b'data')
 
         with self.assertRaises(TypeError):
             resp.body = 123
         self.assertEqual(b'data', resp.body)
         self.assertEqual(4, resp.content_length)
 
+    def test_assign_nonstr_text(self):
+        resp = Response(text='test')
+
+        with self.assertRaises(TypeError):
+            resp.text = b'123'
+        self.assertEqual(b'test', resp.body)
+        self.assertEqual(4, resp.content_length)
+
     def test_send_headers_for_empty_body(self):
         req = self.make_request('GET', '/')
-        resp = Response(req)
+        resp = Response()
 
         self.writer.drain.return_value = ()
         buf = b''
@@ -339,6 +348,7 @@ class TestResponse(unittest.TestCase):
 
         self.writer.write.side_effect = append
 
+        resp.start(req)
         self.loop.run_until_complete(resp.write_eof())
         txt = buf.decode('utf8')
         self.assertRegex(txt, 'HTTP/1.1 200 OK\r\nCONTENT-LENGTH: 0\r\n'
@@ -347,7 +357,7 @@ class TestResponse(unittest.TestCase):
 
     def test_render_with_body(self):
         req = self.make_request('GET', '/')
-        resp = Response(req, b'data')
+        resp = Response(body=b'data')
 
         self.writer.drain.return_value = ()
         buf = b''
@@ -358,6 +368,7 @@ class TestResponse(unittest.TestCase):
 
         self.writer.write.side_effect = append
 
+        resp.start(req)
         self.loop.run_until_complete(resp.write_eof())
         txt = buf.decode('utf8')
         self.assertRegex(txt, 'HTTP/1.1 200 OK\r\nCONTENT-LENGTH: 4\r\n'
@@ -365,10 +376,10 @@ class TestResponse(unittest.TestCase):
                          'DATE: .+\r\nSERVER: .+\r\n\r\ndata')
 
     def test_send_set_cookie_header(self):
-        req = self.make_request('GET', '/')
-        resp = Response(req)
+        resp = Response()
         resp.cookies['name'] = 'value'
 
+        req = self.make_request('GET', '/')
         self.writer.drain.return_value = ()
         buf = b''
 
@@ -378,30 +389,10 @@ class TestResponse(unittest.TestCase):
 
         self.writer.write.side_effect = append
 
+        resp.start(req)
         self.loop.run_until_complete(resp.write_eof())
         txt = buf.decode('utf8')
         self.assertRegex(txt, 'HTTP/1.1 200 OK\r\nCONTENT-LENGTH: 0\r\n'
                          'SET-COOKIE: name=value\r\n'
                          'CONNECTION: keep-alive\r\n'
                          'DATE: .+\r\nSERVER: .+\r\n\r\n')
-
-    def test_explicitly_send_headers(self):
-        req = self.make_request('GET', '/')
-        resp = Response(req)
-
-        self.writer.drain.return_value = ()
-        buf = b''
-
-        def append(data):
-            nonlocal buf
-            buf += data
-
-        self.writer.write.side_effect = append
-
-        resp.send_headers()
-
-        self.loop.run_until_complete(resp.write_eof())
-        txt = buf.decode('utf8')
-        self.assertRegex(txt, 'HTTP/1.1 200 OK\r\nCONTENT-LENGTH: 0\r\n'
-                         'CONNECTION: keep-alive\r\n'
-                         'DATE: .+\r\nSERVER: .+\r\n\r\n')
